"""
Miscellaneous modules for:

* `astro`: astronomy
* `dsp`: signal processing
* `lsd`: misc. operations on arrays, lists, dictionaries and sets
* `stats`: statistical methods
* `plots`: custom plots
* `fermi`: Fermi LAT analysis methods
* `bayes`: Bayesian tools for dealing with posterior distributions
* `grmhd`: tools for dealing with GRMHD numerical simulations

These are modules I wrote which I find useful -- for whatever reason -- in my research.

.. moduleauthor:: Rodrigo Nemmen <rodrigo.nemmen@iag.usp.br>
"""